"""
set up the view for the DIA NN Processor app to do the following tasks,
    1. start a new data analysis queue (view)
    2. process the data (post_processing)
"""
import os
import pandas as pd
import logging

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.core.files.storage import FileSystemStorage
from django.core.files.uploadedfile import InMemoryUploadedFile

from file_manager.models import SampleRecord, DataAnalysisQueue, ProcessingApp


# app name, must be the same as in the database
APPNAME = "DIA NN Wrapper"

# folder to store the methods
APPFOLDER = "media/primary_storage/systemfiles/DIANN/methods/"

# create app folder if not exist
if not os.path.exists(APPFOLDER):
    os.makedirs(APPFOLDER)
logger = logging.getLogger(__name__)

# app view


@ login_required
def view(request):

    args = {
        'SampleRecord':
        SampleRecord.objects.order_by('-pk'),
        'ProcessMethod': [f for f in os.listdir(
            APPFOLDER) if f.endswith('.txt')],
    }

# download link for the processor and its configration files
    processor = ProcessingApp.objects.filter(
        name=APPNAME).first().process_package.name
    if processor:
        args['download_processor'] = processor

# selection of the process configurations from uploaded or
# existing files. save the uploaded files (process) to
# APPFOLDER for future use if save is checked and new method file is uploaded
    if request.method == 'POST' and "delete_config" in request.POST:
        os.remove(APPFOLDER+request.POST.get('DIANN_process_option'))
    elif request.method == 'POST' and "create_new_queue" in request.POST:
        # process method
        if (len(request.FILES) != 0 and
                request.POST.get('DIANN_process_option') == "custom"):
            DIANN_process_method = request.FILES['DIANN_process_file']
            if "Cmd" not in DIANN_process_method.name:
                DIANN_process_method.name = \
                        DIANN_process_method.name.replace(".txt", "_Cmd.txt")
            if request.POST.get('keep_method') == "True":
                fs = FileSystemStorage(location=APPFOLDER)
                # if DIANN_process_method.name does not contain "Cmd",
                # add it and also change DIANN_process_method
                fs.save(DIANN_process_method.name, DIANN_process_method)
        elif request.POST.get('DIANN_process_option') == "None":
            DIANN_process_method = None
        else:
            process_name = request.POST.get('DIANN_process_option')
            process_url = APPFOLDER+process_name
            DIANN_process_method = InMemoryUploadedFile(open(
                process_url, 'r'), None, process_name, None, None, None)
        update_qc = request.POST.get('replace_qc')

        # all the input files
        input_files = {
            "input_file_1": DIANN_process_method,

        }

        newqueue = {
            "processing_name": request.POST.get('analysis_name'),
            'processing_app': ProcessingApp.objects.filter(
                name=APPNAME).first(),
            'process_creator': request.user,
            "update_qc": update_qc,
        }

        # Attach the valid input files to the queue
        for key, value in input_files.items():
            if value:
                newqueue[key] = value

# crate a data analysis queue, attach the sample records to the queue,
# and update the quanlity check.
        newtask = DataAnalysisQueue.objects.create(**newqueue, )
        for item in request.POST.getlist('rawfile_id'):
            newtask.sample_records.add(
                SampleRecord.objects.filter(pk=item).first())
        if update_qc == "True":
            for item in newtask.sample_records.all():
                SampleRecord.objects.filter(pk=item.pk).update(
                    quanlity_check=newtask.pk)

# render the page
    return render(request,
                  'filemanager/DIA_NN_processor.html', args)


def post_processing(queue_id):
    """_this function starts once 3rd party app finished, can be used to
    extract information for the QC numbers, etc._
    """
    analysis_queue = DataAnalysisQueue.objects.filter(pk=queue_id).first()

    # proteins, precursors, ms1 signals, and fwhm from the report.stats.tsv,
    # outputs3
    if analysis_queue.output_file_3:

        try:
            df = pd.read_csv(analysis_queue.output_file_3.path,
                             sep='\t')
            max_index = df["Proteins.Identified"].idxmax()
            analysis_queue.output_QC_number_1 = df.iloc[max_index][
                "Proteins.Identified"]
            analysis_queue.output_QC_number_2 = df.iloc[max_index][
                "FWHM.RT"]*60*1000
            analysis_queue.output_QC_number_3 = df.iloc[max_index][
                "Precursors.Identified"]
            analysis_queue.output_QC_number_4 = df.iloc[max_index][
                "MS1.Signal"]/1000000
        except KeyError:
            logger.error("Invalid output 4 file")
    else:
        analysis_queue.output_QC_number_1 = 0
    analysis_queue.save()
